library(tidyverse)
library(Symobio)
library(sf)
library(rnaturalearth)

sf::sf_use_s2(use_s2 = FALSE)

# Load range maps
range_maps = st_read(
  "~/share/groups/mas_data/Saved_Data_Dropbox_Business/Datensätze/Range Maps/IUCN_range_maps_mammals_version2016/TERRESTRIAL_MAMMALS.shp",
  geometry_column = "geometry",
  promote_to_multi = T
) %>% 
  dplyr::filter(!legend %in% c("Extinct", "Not Mapped"))

# Find species that intersect with Amazon
amazon = st_read(
  "data/geospatial/amazonia_polygons/amazonia_polygons.shp",
  geometry_column = "geometry",
  promote_to_multi = T
) 
range_maps = st_filter(range_maps, amazon)

# Match names against GBIF backbone
range_maps_names_matched = lapply(unique(range_maps$binomial), function(name){
  tryCatch({
    match_result = Symobio::gbif_match_name(name = name)
    if(match_result$status != "ACCEPTED"){
      match_result = gbif_match_name(usageKey = match_result$acceptedUsageKey)
    }
    
    name_matched = if("species" %in% names(match_result)) match_result$species else NA
    data.frame(name_orig = name, name_matched = name_matched)
  }, error = function(e){
    return(NULL)
  })
})
save(range_maps_names_matched, file = "data/r_objects/range_maps_names_matched.RData")

# Subset range maps to target species and focal region
names_matched = Filter(is.data.frame, range_maps_names_matched) %>% 
  bind_rows()

range_maps = range_maps %>% 
  inner_join(names_matched, by = c("binomial" = "name_orig")) %>% 
  group_by(name_matched) %>% 
  summarize(geometry = suppressMessages(st_union(geometry))) 

save(range_maps, file = "data/r_objects/range_maps.RData")

# Gridded range maps
range_maps_gridded = rnaturalearth::ne_countries() %>% 
  dplyr::filter(continent == "South America") %>% 
  sf::st_union() %>% 
  st_make_grid(square = FALSE, cellsize = 1) %>% 
  st_sf() %>% 
  st_join(range_maps, st_intersects, left = F)

save(range_maps_gridded, file = "data/r_objects/range_maps_gridded.RData")
